/*
    oslobike - find free bikes in Oslo.
    Copyright (C) 2019  Harald Eilertsen <haraldei@anduin.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

use reqwest::{
    self,
    header::{
        HeaderMap,
        HeaderValue,
    },
    Client,
};
use crate::station::{Station, StationContainer};
use crate::error::ApiResult;

const API_BASE: &'static str = "https://oslobysykkel.no/api/v1";

fn url_for(endpoint: &str) -> String {
    [API_BASE, endpoint].join("/")
}

pub struct Api {
    client: reqwest::Client,
}

impl Api {
    pub fn new(api_key: String) -> ApiResult<Api> {
        let mut hdrs = HeaderMap::new();
        hdrs.insert("client-identifier", HeaderValue::from_str(&api_key)?);

        let client = Client::builder()
            .default_headers(hdrs)
            .build()?;

        Ok(Api { client })
    }

    pub fn stations(&self) -> ApiResult<Vec<Station>> {
        let response_json = self.client.get(&url_for("stations"))
            .send()?
            .text()?;

        let v: StationContainer = serde_json::from_str(&response_json)?;
        Ok(v.stations)
    }
}
